"""
Transliteration schema base features.
"""

BASE_MAPPING = {
    "а": "a",
    "б": "b",
    "в": "v",
    "г": "g",
    "д": "d",
    "е": "e",
    "з": "z",
    "и": "i",
    "к": "k",
    "л": "l",
    "м": "m",
    "н": "n",
    "о": "o",
    "п": "p",
    "р": "r",
    "с": "s",
    "т": "t",
    "у": "u",
    "ф": "f",
}


class Mapping:
    """
    Letter map for transliteration schema.
    Provides uppercased mapping automatically.
    """

    def __init__(self, mapping: dict):
        self.map = mapping.copy()
        upper_map = {key.capitalize(): value.capitalize() for key, value in mapping.items()}
        self.map.update(upper_map)

    def get(self, key, default=None):
        """Return mapped value for ``key`` if key is in the map, else ``default``."""
        return self.map.get(key, default)

    def __str__(self):
        return str(self.map)

    def __repr__(self):
        return repr(self.map)


class Schema:
    """
    Transliteration schema. Defines the way to translate individual letters.
    """

    def __init__(
        self,
        mapping: dict,
        prev_mapping: dict = None,
        next_mapping: dict = None,
        ending_mapping: dict = None,
    ):
        self.map = Mapping(mapping)
        self.prev_map = Mapping(prev_mapping or {})
        self.next_map = Mapping(next_mapping or {})
        self.ending_map = Mapping(ending_mapping or {})

    def translate_letter(self, prev, curr, next_):
        """Translate ``curr`` letter according to schema mappings.

        ``prev`` (the previous letter) and ``next_`` (the next one)
        are taken into consideration according to corresponding mappings.

        """
        letter = self.prev_map.get(prev + curr)
        if letter is None:
            letter = self.next_map.get(curr + next_)
        if letter is None:
            letter = self.map.get(curr, curr)
        return letter

    def translate_ending(self, ending):
        """Translate word ending according to schema mapping."""
        return self.ending_map.get(ending)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        ("Юлия Щеглова", "Yuliya Shcheglova"),
        ("Гайа Васильева", "Gaya Vasilyeva"),
        ("Андрей Видный", "Andrey Vidnyy"),
        ("Артём Краевой", "Artyem Krayevoy"),
        ("Мадыр Нечёткий", "Madyr Nechetkiy"),
        ("Оксана Клеёнкина", "Oksana Kleyonkina"),
        ("Игорь Ильин", "Igor' Ilyin"),
    ],
)
def test_mvd_782(source, expected):
    assert iuliia.translate(source, iuliia.MVD_782) == expected



import iuliia


def test_schema_names():
    names = iuliia.Schemas.names()
    assert names == [
        "gost_16876",
        "gost_16876_alt",
        "gost_52290",
        "gost_52535",
        "gost_7034",
        "gost_779",
        "gost_779_alt",
        "mvd_310",
        "mvd_310_fr",
        "mvd_782",
        "wikipedia",
        "yandex_maps",
        "yandex_money",
    ]


def test_get_schema_by_name():
    schema = iuliia.Schemas.get("wikipedia")
    assert schema == iuliia.WIKIPEDIA



from iuliia import engine


def test_translate():
    schema = engine.Schema({})
    assert engine.translate("Iuliia", schema) == "Iuliia"


def test_mapping():
    schema = engine.Schema({"a": "1", "i": "2", "l": "3", "u": "4"})
    assert engine.translate("Iuliia", schema) == "243221"


def test_prev_mapping():
    schema = engine.Schema({}, prev_mapping={"li": ""})
    assert engine.translate("Iuliia", schema) == "Iulia"


def test_next_mapping():
    schema = engine.Schema({}, next_mapping={"iu": "y"})
    assert engine.translate("Iuliia", schema) == "Yuliia"


def test_ending_mapping():
    schema = engine.Schema({}, ending_mapping={"ia": "ya"})
    assert engine.translate("Iuliia", schema) == "Iuliya"


def test_short_word():
    schema = engine.Schema({})
    assert engine.translate("Iu", schema) == "Iu"


def test_empty_word():
    schema = engine.Schema({})
    assert engine.translate("", schema) == ""



"""
Transliterate Cyrillic → Latin in every possible way
"""

# flake8: noqa
from enum import Enum
from iuliia.engine import translate
from iuliia.schema import Schema
from iuliia.gost16876 import GOST_16876, GOST_16876_ALT
from iuliia.gost52290 import GOST_52290
from iuliia.gost52535 import GOST_52535
from iuliia.gost7034 import GOST_7034
from iuliia.gost779 import GOST_779, GOST_779_ALT
from iuliia.mvd310 import MVD_310, MVD_310_FR
from iuliia.mvd782 import MVD_782
from iuliia.wikipedia import WIKIPEDIA
from iuliia.yandexmaps import YANDEX_MAPS
from iuliia.yandexmoney import YANDEX_MONEY

__version__ = "0.6.0"
__all__ = []


class Schemas(Enum):
    """All supported transliteration schemas."""

    gost_16876 = GOST_16876
    gost_16876_alt = GOST_16876_ALT
    gost_52290 = GOST_52290
    gost_52535 = GOST_52535
    gost_7034 = GOST_7034
    gost_779 = GOST_779
    gost_779_alt = GOST_779_ALT
    iso_9 = GOST_779
    mvd_310 = MVD_310
    mvd_310_fr = MVD_310_FR
    mvd_782 = MVD_782
    wikipedia = WIKIPEDIA
    yandex_maps = YANDEX_MAPS
    yandex_money = YANDEX_MONEY

    @classmethod
    def names(cls):
        """Return sorted list of all supported schemas."""
        return sorted(item.name for item in cls)

    @classmethod
    def get(cls, name):
        """Return schema by its name or ``None`` if nothing found."""
        item = cls.__members__.get(name)
        return item.value if item else None



"""
GOST R 7.0.34-2014 transliteration schema.
http://localhost:3000/iuliia/gost-7034/
"""

from .schema import Schema, BASE_MAPPING

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "yo",
        "ж": "zh",
        "й": "j",
        "х": "x",
        "ц": "c",
        "ч": "ch",
        "ш": "sh",
        "щ": "shh",
        "ъ": "''",
        "ы": "y",
        "ь": "'",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

GOST_7034 = Schema(MAPPING)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossiya, gorod Yoshkar-Ola, ulitsa Yana Krastynya",
        ),
        ("Ельцин", "Yeltsin"),
        ("Раздольное", "Razdolnoye"),
        ("Юрьев", "Yuryev"),
        ("Белкин", "Belkin"),
        ("Бийск", "Biysk"),
        ("Подъярский", "Podyarsky"),
        ("Мусийкъонгийкоте", "Musiykyongiykote"),
        ("Давыдов", "Davydov"),
        ("Усолье", "Usolye"),
        ("Выхухоль", "Vykhukhol"),
        ("Дальнегорск", "Dalnegorsk"),
        ("Ильинский", "Ilyinsky"),
        ("Красный", "Krasny"),
        ("Великий", "Veliky"),
        ("Набережные Челны", "Naberezhnye Chelny"),
    ],
)
def test_wikipedia(source, expected):
    assert iuliia.translate(source, iuliia.WIKIPEDIA) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        ("Юлия Щеглова", "Yuliya Shcheglova"),
        ("Гайа Васильева", "Gaya Vasilyeva"),
        ("Андрей Видный", "Andrey Vidnyy"),
    ],
)
def test_mvd_310(source, expected):
    assert iuliia.translate(source, iuliia.MVD_310) == expected


@pytest.mark.parametrize(
    "source,expected",
    [
        ("Юлия Щеглова", "Iouliia Chtcheglova"),
        ("Гайа Васильева", "Gaia Vasilieva"),
        ("Андрей Видный", "Andrei Vidnyi"),
        ("Оксана Снегирёва", "Oxana Sneguireva"),
        ("Юрий Васин", "Iourii Vasine"),
    ],
)
def test_mvd_310_fr(source, expected):
    assert iuliia.translate(source, iuliia.MVD_310_FR) == expected



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossiya, gorod Yoshkar-Ola, ulitsa Yana Krastynya",
        ),
        ("Санкт-Петербург, Подъездной пер", "Sankt-Peterburg, Podyezdnoy per"),
        ("Москва, ул Подъёмная", "Moskva, ul Podyomnaya"),
        ("Астрахань, ул Подъяпольского", "Astrakhan, ul Podyapolskogo"),
        ("Щегловитовка", "Scheglovitovka"),
        ("Новый Уренгой", "Noviy Urengoy"),
    ],
)
def test_yandex_maps(source, expected):
    assert iuliia.translate(source, iuliia.YANDEX_MAPS) == expected



"""
Transate engine.
"""

from iuliia.schema import Schema


def translate(source: str, schema: Schema):
    """
    Translate source Cyrillic string into Latin using specified schema.
    Translates sentences word by word, delegating specifics of transliteration
    to specified schema.
    """
    translated = (_translate_word(word, schema) for word in source.split())
    return " ".join(translated)


def _translate_word(word: str, schema: Schema):
    stem, ending = _split_word(word)
    translated_ending = schema.translate_ending(ending) if ending else None
    if translated_ending:
        translated = _translate_letters(stem, schema)
        translated.append(translated_ending)
    else:
        translated = _translate_letters(word, schema)
    return "".join(translated)


def _translate_letters(word, schema):
    translated = []
    for prev, curr, next_ in _letter_reader(word):
        letter = schema.translate_letter(prev, curr, next_)
        translated.append(letter)
    return translated


def _split_word(word):
    ending_length = 2
    if len(word) > ending_length:
        stem = word[:-ending_length]
        ending = word[-ending_length:]
    else:
        stem = word
        ending = ""
    return stem, ending


def _letter_reader(stem):
    idx = 0
    prev = ""
    curr = ""
    next_ = ""
    # pylint: disable=C0200
    for idx in range(0, len(stem)):
        if curr != "":
            prev = curr
        curr = next_ or stem[idx]
        if idx < len(stem) - 1:
            next_ = stem[idx + 1]
        else:
            next_ = ""
        yield prev, curr, next_



"""
Yandex.Money transliteration schema.
https://dangry.ru/iuliia/yandex-money/
"""

from .schema import Schema, BASE_MAPPING

MAPPING = {
    **BASE_MAPPING,
    **{
        "ё": "e",
        "ж": "zh",
        "й": "i",
        "х": "kh",
        "ц": "ts",
        "ч": "ch",
        "ш": "sh",
        "щ": "sch",
        "ъ": "",
        "ы": "y",
        "ь": "",
        "э": "e",
        "ю": "yu",
        "я": "ya",
    },
}

YANDEX_MONEY = Schema(MAPPING)



import pytest
import iuliia


@pytest.mark.parametrize(
    "source,expected",
    [
        (
            "Россия, город Йошкар-Ола, улица Яна Крастыня",
            "Rossiya, gorod Yoshkar-Ola, ulitsa Yana Krastynya",
        ),
        (
            "Юлия, съешь ещё этих мягких французских булок из Йошкар-Олы, да выпей алтайского чаю",
            "Yuliya, syesh' eshche etikh myagkikh frantsuzskikh bulok iz Yoshkar-Oly, "
            "da vypey altayskogo chayu",
        ),
        ("Ё Крё Мякоё", "Yo Krye Myakoyo"),
    ],
)
def test_gost_52290(source, expected):
    assert iuliia.translate(source, iuliia.GOST_52290) == expected



import pytest
from iuliia.schema import Mapping


@pytest.fixture
def mapping():
    return Mapping({"a": "x", "b": "y", "cd": "zz"})


def test_map(mapping):
    assert mapping.map == {"a": "x", "b": "y", "cd": "zz", "A": "X", "B": "Y", "Cd": "Zz"}


def test_get(mapping):
    assert mapping.get("a") == "x"
    assert mapping.get("B") == "Y"
    assert mapping.get("f") is None
    assert mapping.get("g", "g") == "g"


def test_str(mapping):
    assert str(mapping) == str(mapping.map)
    assert repr(mapping) == repr(mapping.map)
